/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: temp_server.c,v 1.18.4.1 2002/03/03 11:49:17 lloydy Exp $
 *
 */
#include "temp_server.h"
#include "ircd.h"
#include "enpress.h"

/* temp_server.c - Routines to handle a SERVER command from a 
 * peer in the temporary list */

int temp_server(struct tempinfo *temp,Cmd cmd)
{
	/* params[0] is the server name
	 * params[1] is the hopcount, 0 in this case
	 * params[2] is the token, not sure we need it
	 * text is the server info field */
	struct nline *nline=NULL;
	
	/* Check that this peer has successfully used PASS */
	if(!(temp->flags & USR_PASS)) {
		force_quit(temp->fd,"You are trying to establish a server link from an unassured host");
		return 0;
	}
	
	
	if(!check_params(cmd,1)) {
/* 		printf("Not enough params\n"); */
		return 0;
	}
	
	if(cmd->text) {
		/* Check for duplicates */
		if(server_locate(cmd->params[0])) {
			/* Dup server */
			fd_list[temp->fd].temp=NULL;
			if(temp->fd==highest_fd) 
				highest_fd=-1;
			temp_rem(temp->fd);
			close(temp->fd);
			return 0;
		}

		/* Check against nline */
		nline=get_nline(temp->host);
		if(nline==NULL || lc_strncmp(nline->serv_name,cmd->params[0],-1)!=0 ) {
			force_quit(temp->fd,"Your host isn't authorised");
			return 0;
		}
		
		strncpy(temp->info,cmd->text,SERVINFO_MAX);
		strncpy(temp->name,cmd->params[0],SERVNAME_MAX);
		temp->flags |= USR_SERVER;
		send_temp(temp,"CAPAB :TS4");
	} else {
		/* printf("Not enough params\n");  */
	}
		
	return 1;
}

/*
* Deal with PASS in temp state
*/
int temp_pass(struct tempinfo *t_ptr,Cmd cmd)
{
	/* Format */
	/* PASS <password> <version> <flags> [other stuff] */
	/* We don't have any options/flags atm :) */	
	/* So we just need to check the password and the version is 0.94d+ cmircd */
	char *pass=NULL;
	struct nline *nline=NULL;
	
	if(!(cmd->params[0]) && !(cmd->text)) {
		return 0;
	}

	if(cmd->params[0]) {
		pass=cmd->params[0];
	} else {
		pass=cmd->text;
	}
	
	/* Get the NLINE for this host */
	nline=get_nline(t_ptr->host);
	
	if(nline) {
		if( strcmp(nline->password,cmd->params[0])==0 ) {
			t_ptr->flags |= USR_PASS;
		}
	} else {
	}
		
	return 1;
}


/*
* Deal with CAPAB required by the TS4
* Protocol
*/
int temp_capab(struct tempinfo *t_ptr,Cmd cmd)
{
	int i,last,curr=0,len;
	char temp[300];
	char *capabs[4]={NULL,NULL,NULL,NULL};
#ifdef USE_OPENSSL
	EncryptInfo ctx=NULL;
	struct nline *n_ptr=NULL;
#endif

	/* We need cmd->text */
	if(!cmd->text) {
		return 0;
	}
	
	/* Split it up, set spaces to \0 */
	len=strlen(cmd->text);
	strncpy(temp,cmd->text,len);
	temp[strlen(cmd->text)]='\0';
	last=0;
	for(i=0;i<=len && curr < 4;i++) {
		if(temp[i]==' ' || temp[i]=='\0') {
			capabs[curr++]=&(temp[last]);
			temp[i]='\0';
			i++;
			last=i;
		}
	}

	for(i=0;capabs[i]!=NULL;i++) {
		printf("Looking at %s\n",capabs[i]);
		if(lc_strncmp(capabs[i],"ZIP",-1)==0) {
			/* Set SERVER_ZIP flag */
			t_ptr->flags |= USR_ZIP;
		} 
		if(lc_strncmp(capabs[i],"TS4",-1)==0) {
			/* Set SERVER_TS4 flags */
			t_ptr->flags |= USR_TS4;
		}
#ifdef USE_OPENSSL
		if(lc_strncmp(capabs[i],"ENCRYPT",-1)==0) {
			printf("Matched encrypt\n");	
			if( !(t_ptr->ctx) ) {
				/* Server wants encryption, use key from the file */
				/* Get NLINE for the key */
				n_ptr=get_nline(t_ptr->host);
				/* Try and get context to encrypt */
				ctx = generate_key( n_ptr?n_ptr->key:(unsigned char *) "*" );
				printf("Gone encrypted with ctx=%x\n",ctx);
				if(ctx) {
					/* we can go encrypted */
					send_temp(t_ptr,"CAPAB :ENCRYPT");
					t_ptr->ctx=ctx;
				}
			}
				
		}
#endif
	}

	return 1;
}
