/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_kill.c,v 1.6.4.3 2002/03/05 22:17:32 lloydy Exp $
 *
 */
#include "chunkyircd.h"
#include "client.h"

CMIRCD_MODULE(1.01,CMD_CLIENT | CMD_SERVER);

/* 
* Check the KILL request is valid then call client_kill to 
* kill the client
*/
int cmd_kill(ClientInfo c_ptr,ServerInfo s_ptr,Cmd cmd) 
{
	ClientInfo k_ptr=NULL;
	ClientInfo sender=NULL;
	ServerInfo server=NULL;
	char buffer[BUF_SIZE];
	
	if(c_ptr) {

		/* Check for irc ops */
		if( !(c_ptr->mode & MODE_IRCOP) ) {
			send_numeric(c_ptr,get_my_server(),481,":Permission Denied - You're not an IRC Op");
			 
			return 1;
		}
		
		/* Check we have enough parameters */
		if(!check_params(cmd,1) || !cmd->text) {
			send_numeric(c_ptr,get_my_server(),461,"KILL: Not enough parameters");
			 
			return 1;
		}
		
		/* Check the requested client exists */
		k_ptr=client_locate(cmd->params[0]);
		if(!k_ptr) {
			/* Client doesn't exist */
			ircd_sprintf(buffer,BUF_SIZE,"%s :No such nick/channel",cmd->params[0]);
			send_numeric(c_ptr,get_my_server(),401,buffer);
			 
			return 1;
		}

		/* Inform the other servers */
		client_kill(s_ptr,NULL,c_ptr,k_ptr,cmd->text);
		
	} 
	if(s_ptr && cmd->origin) {
		
		/* Check the requested client exists */
		k_ptr=client_locate(cmd->params[0]);
		if(!k_ptr) {
			/* Client doesn't exist */
			 
			return 1;
		}

		/* Inform the other servers */
		if((server=server_locate(cmd->origin))) {
			client_kill(s_ptr,server,NULL,k_ptr,cmd->text);
		}
		if((sender=client_locate(cmd->origin))) {
			client_kill(s_ptr,NULL,sender,k_ptr,cmd->text);
		}
	}
		
	 
	return 1;
}

	
