/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_privmsg.c,v 1.20.4.4 2002/03/05 22:54:04 lloydy Exp $
 *
 */
#include "chunkyircd.h"
#include "channel.h"
#include "network.h"

/* Export version info */
CMIRCD_MODULE(1.01,CMD_CLIENT | CMD_SERVER);

static int privmsg_to_channel(ServerInfo ,ClientInfo ,ChannelInfo ,char *);
static int privmsg_to_user(ClientInfo,ClientInfo,char *);

/* cmd_privmsg - deals with privmsg system */
int cmd_privmsg(ClientInfo c_ptr,ServerInfo s_ptr,Cmd cmd)
{
	ClientInfo recip=NULL;
	ChannelInfo chan=NULL;
	ClientInfo sender=NULL;
	char buffer[BUF_SIZE];
	
	/* find out if it's a client or a room */
	if(!cmd->params[0] || !cmd->text) {
		ircd_sprintf(buffer,BUF_SIZE,"%s : Not enough parameters\n",cmd->command);
		send_numeric(c_ptr,get_my_server(),461,buffer);
		 
		return 0;
	}

	/* Try and get real origin */
        if(!c_ptr) {
		sender = client_locate(cmd->origin);

		if(!sender) {
			return 0;
		}
	} else {
		sender = c_ptr;
	}

	/* Try and match channel first */
	chan = locate_channel(cmd->params[0]);
	if(chan) {
		/* Message to channel */
		privmsg_to_channel(s_ptr,sender,chan,cmd->text);
		sender->idle_timer = time(NULL);
		return 1;
	}
	
	/* Try and match a  user */
	recip = client_locate(cmd->params[0]);
	if(recip) {
		/* Message to client */
		privmsg_to_user(sender,recip,cmd->text);
		sender->idle_timer = time(NULL);
		return 1;
	}
	
	/* Attempt to match by a server mask */
	if( (cmd->params[0][0]=='$') && (sender->mode & MODE_IRCOP) ) {
		/* Send to servers */
		ircd_sprintf(buffer,BUF_SIZE,"PRIVMSG %s :%s",cmd->params[0],cmd->text);
		servers_relay(s_ptr,sender->nick,buffer);
		/* Send to local clients */
		client_msg_by_server_mask(sender,&(cmd->params[0][1]),"PRIVMSG",cmd->text,NULL);
		return 1;
	}

	/* Send using hostmasks */
	if(cmd->params[0][0]=='#' && sender->mode & MODE_IRCOP) {
		/* Send to servers */
		ircd_sprintf(buffer,BUF_SIZE,"PRIVMSG %s :%s",cmd->params[0],cmd->text);
		servers_relay(s_ptr,sender->nick,buffer);
		/* Local Clients */
		client_msg_by_host_mask(sender,&(cmd->params[0][1]),"PRIVMSG",cmd->text,NULL);
		return 1;
	} 
	
	/* Send error if its a local client */
	if(c_ptr) {
		ircd_sprintf(buffer,BUF_SIZE,"%s :No such nick/channel",cmd->params[0]);
		send_numeric(sender,get_my_server(),401,buffer);
	}
	return 1;

}

/* 
 * send a privmsg to a user 
 */
static int privmsg_to_user(ClientInfo sender,
			   ClientInfo recip,
			   char *text)
{
	char cmd_buff[500];
	
	/* form end of string */
	if(text==NULL) {
		return 0;
	} else {
		if( (recip->mode & MODE_AWAY) && (recip->server==get_my_server()) ) {
			ircd_sprintf(cmd_buff,BUF_SIZE,"%s :%s",recip->nick,recip->away_msg);
			send_numeric(sender,get_my_server(),301,cmd_buff);
		}

		ircd_sprintf(cmd_buff,BUF_SIZE,"PRIVMSG %s :%s",recip->nick,text);
		send_client_client(recip,sender,cmd_buff);
	
		/* call procedure to send to client */
	 	return 1;
	}
}

/* 
 * send a privmsg to a channel 
 */
static int privmsg_to_channel(ServerInfo s_ptr,
			      ClientInfo sender,
			      ChannelInfo chan,
			      char *text)
{
	/* Max size of irc command */
	char buffer[512];
	ChannelMember curr_client=NULL;
	int rand_no;
	
	if(!channel_msg_check_perms(chan,sender) ) {
		/* Tell them they don't have permission */
		ircd_sprintf(buffer,BUF_SIZE,"%s :Cannot send to channel",chan->name);
		send_numeric(sender,get_my_server(),404,buffer);
		return 1;
	}
	
	/* Keep an eye open for buffer overflows */
	if(!ircd_sprintf(buffer,BUF_SIZE,"PRIVMSG %s :%s",chan->name,text)) 
		return 0;

	/* Send to the clients */
	rand_no=rand() % 1000 +1;
	
	curr_client=chan->users_start;
	while(curr_client) {
		if(curr_client->user_info != sender) {
			if(curr_client->user_info->fd==-1) {
				/* Remote client */
				if(curr_client->user_info->server->rand_no!=rand_no) {
					/* We've not already sent it */
					if(curr_client->user_info->server != s_ptr 
							&& curr_client->user_info->server->fd!=-1) {
						send_origin_server(curr_client->user_info->server,sender->nick,buffer);
						curr_client->user_info->server->rand_no=rand_no;
					}
				}
		
			} else {
				send_client_client(curr_client->user_info,sender,buffer);
			}
		}
		curr_client=curr_client->next;
	}
		
			
	return 1;
}

